#!/usr/bin/env python
# coding=utf-8

"""the request type in tencent qcloud cos"""

from cos_params_check import ParamCheck
import collections

################################################################################
# BaseRequest基本类型的请求                                                    #
################################################################################
class BaseRequest(object):
    # bucket_name: bucket的名称
    # cos_path: cos的绝对路径, 即从bucket下的根/开始
    def __init__(self, bucket_name, cos_path):
        self._bucket_name  = bucket_name.strip()
        self._cos_path     = cos_path.strip()
        self._param_check   = ParamCheck()

    # 设置bucket_name
    def set_bucket_name(self, bucket_name=u''):
        self._bucket_name = bucket_name.strip()

    # 获取bucket_name
    def get_bucket_name(self):
        return self._bucket_name

    # 设置cos_path
    def set_cos_path(self, cos_path=u''):
        self._cos_path = cos_path.strip()
    
    # 获取cos_path
    def get_cos_path(self):
        return self._cos_path

    # 获取错误信息
    def get_err_tips(self):
        return self._param_check.get_err_tips()

    # 检查参数是否合法
    def check_params_valid(self):
        if not self._param_check.check_param_unicode('bucket', self._bucket_name):
            return False
        return self._param_check.check_param_unicode('cos_path', self._cos_path)

################################################################################
# CreateFolderRequest  创建目录类型的请求                                      #
################################################################################
class CreateFolderRequest(BaseRequest):
    # bucket_name: bucket的名称
    # cos_path:    cos的绝对路径, 从bucket根/开始
    # biz_attr:    目录的属性
    def __init__(self, bucket_name, cos_path, biz_attr=u''):
        super(CreateFolderRequest, self).__init__(bucket_name, cos_path)
        self._biz_attr = biz_attr

    # 设置biz_attr
    def set_biz_attr(self, biz_attr):
        self._biz_attr = biz_attr

    # 获取biz_attr
    def get_biz_attr(self):
        return self._biz_attr

    # 检查参数是否合法
    def check_params_valid(self):
        if not super(CreateFolderRequest, self).check_params_valid():
            return False
        if not self._param_check.check_param_unicode('biz_attr', 
                self._biz_attr):
            return False
        if not self._param_check.check_cos_path_valid(self._cos_path, 
                is_file_path=False):
            return False
        return self._param_check.check_not_cos_root(self._cos_path)

################################################################################
# UploadFileRequest  单文件上传请求                                            #
################################################################################
class UploadFileRequest(BaseRequest):
    # bucket_name:            bucket的名称
    # cos_path:               cos的绝对路径(目的路径), 从bucket根/开始
    # local_path:             上传的本地文件路径(源路径)
    # biz_attr:               文件的属性
    # insert_only:            是否覆盖写, 0覆盖, 1不覆盖,返回错误

    def __init__(self, bucket_name, cos_path, local_path, biz_attr=u'',
            insert_only=1):
        super(UploadFileRequest, self).__init__(bucket_name, cos_path)
        self._local_path          = local_path.strip()
        self._biz_attr            = biz_attr
        self._insert_only         = insert_only

    # 设置local_path
    def set_local_path(self, local_path):
        self._local_path = local_path.strip()

    # 获取local_path
    def get_local_path(self):
        return self._local_path

    # 设置biz_attr
    def set_biz_attr(self, biz_attr):
        self._biz_attr = biz_attr

    # 获取biz_attr
    def get_biz_attr(self):
        return self._biz_attr

    # 设置insert_only，0表示如果文件存在, 则覆盖
    def set_insert_only(self, insert_only):
        self._insert_only = insert_only

    # 获取insert_only
    def get_insert_only(self):
        return self._insert_only

    # 检查参数是否有效 
    def check_params_valid(self):
        if not super(UploadFileRequest, self).check_params_valid():
            return False
        if not self._param_check.check_cos_path_valid(self._cos_path, 
                is_file_path=True):
            return False
        if not self._param_check.check_param_unicode('biz_attr',
                self._biz_attr):
            return False
        if not self._param_check.check_param_unicode('local_path',
                self._local_path):
            return False
        if not self._param_check.check_local_file_valid(self._local_path):
            return False
        if not self._param_check.check_param_int('insert_only', 
                self._insert_only):
            return False
        return self._param_check.check_insert_only(self._insert_only)

################################################################################
# UploadSliceFileRequest  分片文件上传请求                                     #
################################################################################
class UploadSliceFileRequest(UploadFileRequest):
    # bucket_name:  bucket的名称
    # cos_path:     cos的绝对路径(目的路径), 从bucket根/开始
    # local_path:   上传的本地文件路径(源路径)
    # biz_attr:     文件的属性
    # slice_size:   分片大小(字节, 默认512KB)
    def __init__(self, bucket_name, cos_path, local_path, slice_size=512*1024,
            biz_attr=u''):
        super(UploadSliceFileRequest, self).__init__(bucket_name, cos_path, local_path,
                biz_attr)
        self._slice_size = slice_size

    # 设置分片大小
    def set_slice_size(self, slice_size):
        self._slice_size = slice_size

    # 获取分片大小
    def get_slice_size(self):
        return self._slice_size

    # 检查参数是否有效 
    def check_params_valid(self):
        if not super(UploadSliceFileRequest, self).check_params_valid():
            return False
        return self._param_check.check_slice_size(self._slice_size)

################################################################################
# UpdateFolderRequest 更新目录请求                                             #
################################################################################
class UpdateFolderRequest(BaseRequest):
    # biz_attr:               目录属性 
    def __init__(self, bucket_name, cos_path, biz_attr=u''):
        super(UpdateFolderRequest, self).__init__(bucket_name, cos_path)
        self._biz_attr = biz_attr

    # 设置biz_attr
    def set_biz_attr(self, biz_attr):
        self._biz_attr = biz_attr

    # 获取biz_attr
    def get_biz_attr(self):
        return self._biz_attr

    # 检查参数是否有效 
    def check_params_valid(self):
        if not super(UpdateFolderRequest, self).check_params_valid():
            return False
        if not self._param_check.check_cos_path_valid(self._cos_path, 
                is_file_path=False):
            return False
        if not self._param_check.check_not_cos_root(self._cos_path):
            return False
        return self._param_check.check_param_unicode('biz_attr',
                self._biz_attr)

################################################################################
# UpdateFileRequest 更新文件请求                                               #
################################################################################
class UpdateFileRequest(BaseRequest):
    # bucket_name:  bucket的名称
    # cos_path:     cos的绝对路径, 从bucket根/开始
    # biz_attr:     要更新的文件的属性
    # authority:              文件权限:
    #                         eInvalid(继承bucket), 
    #                         eWRPrivate(私有读写),
    #                         eWPrivateRPublic(私有写, 公有读)
    # customer_header:        用户自定义的HTTP请求头,包括以下成员
    # cache_control:          文件的缓存机制,参见HTTP的Cache-Control
    # content_type:           文件的MIME信息,参见HTTP的Content-Type
    # content_disposition:    MIME协议的扩展,参见HTTP的Content-Disposition
    # content_language:       文件的语言, 参见HTTP的Content-Language
    # _x_cos_meta_dict:       用户自定义的属性, key是以x-cos-meta-开头,value为属性值
    def __init__(self, bucket_name, cos_path):
        super(UpdateFileRequest, self).__init__(bucket_name, cos_path)
        self._flag                = 0
        self._biz_attr            = u''
        self._custom_headers      = {}
        self._authority           = u''
        self._cache_control       = u''
        self._content_type        = u''
        self._content_disposition = u''
        self._content_language    = u''
        self._x_cos_meta_dict     = {}

    # 获取flag
    def get_flag(self):
        return self._flag

    # 设置biz_attr
    def set_biz_attr(self, biz_attr):
        self._biz_attr = biz_attr
        self._flag |= 0x01

    # 获取biz_attr
    def get_biz_attr(self):
        return self._biz_attr

    # 设置authority, 合法取值如下所示
    # eInvalid(继承bucket), 
    # eWRPrivate(私有读写),
    # eWPrivateRPublic(私有写, 公有读)
    def set_authority(self, authority):
        self._authority = authority
        self._flag |= 0x80

    # 获取authority
    def get_authority(self):
        return self._authority

    # 设置缓存机制Cache-Control
    def set_cache_control(self, cache_control):
        self._cache_control = cache_control
        self._flag |= 0x40
        self._custom_headers[u'Cache-Control'] = cache_control

    # 设置Content-Type
    def set_content_type(self, content_type):
        self._content_type = content_type
        self._flag |= 0x40
        self._custom_headers['Content-Type'] = content_type

    # 设置Content-Disposition
    def set_content_disposition(self, content_disposition):
        self._content_disposition = content_disposition
        self._flag |= 0x40
        self._custom_headers['Content-Disposition'] = content_disposition

    # 设置Content-Language
    def set_content_language(self, content_language):
        self._content_language = content_language
        self._flag |= 0x40
        self._custom_headers['Content-Language'] = content_language

    # 设置自定义的x-cos-meta, key以x-cos-meta-开头
    # 例如自定义key为u'x-cos-meta-len', value为u'1024'
    def set_x_cos_meta(self, key, value):
        self._flag |= 0x40
        self._x_cos_meta_dict[key] = value
        self._custom_headers[key] = value

    # convert a dict's keys & values from `unicode` to `str`
    def _convert_dict(self, data):
         if isinstance(data, basestring):
             return str(data)
         elif isinstance(data, collections.Mapping):
             return dict(map(self._convert_dict, data.iteritems()))
         elif isinstance(data, collections.Iterable):
             return type(data)(map(self._convert_dict, data))
         else:
             return data

    # 获取自定义的HTTP头
    def get_custom_headers(self):
        return self._convert_dict(self._custom_headers)

    # 检查参数是否合法
    def check_params_valid(self):
        if not super(UpdateFileRequest, self).check_params_valid():
            return False
        if not self._param_check.check_cos_path_valid(self._cos_path, 
                is_file_path=True):
            return False
        if not self._param_check.check_update_flag(self._flag):
            return False
        if not self._param_check.check_param_unicode('biz_attr', 
                self._biz_attr):
            return False
        if not self._param_check.check_param_unicode('authority', 
                self._authority):
            return False
        if not self._param_check.check_file_authority(self._authority):
            return False
        if not self._param_check.check_param_unicode('cache_control', 
                self._cache_control):
            return False
        if not self._param_check.check_param_unicode('content_type',
                self._content_type):
            return False
        if not self._param_check.check_param_unicode('content_disposition',
                self._content_disposition):
            return False
        if not self._param_check.check_param_unicode('content_language',
                self._content_language):
            return False
        return self._param_check.check_x_cos_meta_dict(self._x_cos_meta_dict)

################################################################################
# MoveFileRequest 移动文件请求                                                 #
################################################################################
class MoveFileRequest(BaseRequest):
    # bucket_name:  bucket的名称
    # src_cos_path: 要移动的文件源路径
    # dst_cos_path: 要移动到文件目的地路径
    # to_over_write:   是否覆盖, 0(默认): 不覆盖, 1: 覆盖
    def __init__(self, bucket_name, src_cos_path, dst_cos_path, to_over_write = 0):
        super(MoveFileRequest, self).__init__(bucket_name, src_cos_path)
        self._src_cos_path  = src_cos_path
        self._dst_cos_path  = dst_cos_path
        self._to_over_write = to_over_write

    # 设置src_cos_path
    def set_src_cos_path(self, src_cos_path):
        self._src_cos_path = src_cos_path

    # 获取src cos path
    def get_src_cos_path(self):
        return self._src_cos_path

    # 设置dst_cos_path
    def set_src_cos_path(self, dst_cos_path):
        self._dst_cos_path = dst_cos_path

    # 获取dst_cos_path
    def get_dst_cos_path(self):
        return self._dst_cos_path

    # 设置over_write标志
    def set_over_write(self, to_over_write):
        self._to_over_write = to_over_write

    # 获取over_write标志
    def get_over_write(self):
        return self._to_over_write

    # 检查参数是否合法
    def check_params_valid(self):
        if not super(MoveFileRequest, self).check_params_valid():
            return False
        if not self._param_check.check_cos_path_valid(self._src_cos_path, 
                is_file_path=True):
            return False
        if not self._param_check.check_param_unicode('dst_cos_path', 
            self._dst_cos_path):
            return False
        if not self._param_check.check_cos_path_valid(self._dst_cos_path, 
                is_file_path=True):
            return False
        if not self._param_check.check_param_int('over_write', 
            self._to_over_write):
            return False
        return self._param_check.check_move_over_write(self._to_over_write)

################################################################################
# StatRequest 获取文件属性请求                                                 #
################################################################################
class StatFileRequest(BaseRequest):
    # bucket_name:  bucket的名称
    # cos_path:     cos的文件路径, 从bucket根/开始, 不以/结束
    def __init__(self, bucket_name, cos_path):
        super(StatFileRequest, self).__init__(bucket_name, cos_path)

    # 检查参数是否合法
    def check_params_valid(self):
        if not super(StatFileRequest, self).check_params_valid():
            return False
        return self._param_check.check_cos_path_valid(self._cos_path, 
                is_file_path=True)

################################################################################
# StatRequest 获取目录属性请求                                                 #
################################################################################
class StatFolderRequest(BaseRequest):
    # bucket_name:  bucket的名称
    # cos_path:     cos的目录路径, 从bucket根/开始, 以/结束
    def __init__(self, bucket_name, cos_path):
        super(StatFolderRequest, self).__init__(bucket_name, cos_path)

    # 检查参数是否合法
    def check_params_valid(self):
        if not super(StatFolderRequest, self).check_params_valid():
            return False
        return self._param_check.check_cos_path_valid(self._cos_path, 
                is_file_path=False)

################################################################################
# DelFileRequest 删除文件请求                                                  #
################################################################################
class DelFileRequest(BaseRequest):
    # bucket_name:  bucket的名称
    # cos_path:     cos的文件路径, 从bucket根/开始, 不以/结束
    def __init__(self, bucket_name, cos_path):
        super(DelFileRequest, self).__init__(bucket_name, cos_path)

    # 检查参数是否合法
    def check_params_valid(self):
        if not super(DelFileRequest, self).check_params_valid():
            return False
        return self._param_check.check_cos_path_valid(self._cos_path,
                is_file_path=True)

################################################################################
# DelFolderRequest 删除目录请求                                                #
################################################################################
class DelFolderRequest(BaseRequest):
    # bucket_name:  bucket的名称
    # cos_path:     cos的目录路径, 从bucket根/开始, 以/结束
    def __init__(self, bucket_name, cos_path):
        super(DelFolderRequest, self).__init__(bucket_name, cos_path)

    # 检查参数合法
    def check_params_valid(self):
        if not super(DelFolderRequest, self).check_params_valid():
            return False
        if not self._param_check.check_cos_path_valid(self._cos_path, 
                is_file_path=False):
            return False
        return self._param_check.check_not_cos_root(self._cos_path)

################################################################################
# ListFolderRequest 获取目录列表的请求                                         #
################################################################################
class ListFolderRequest(BaseRequest):
    # bucket_name:  bucket的名称
    # cos_path:     cos的绝对路径, 从bucket根/开始
    # num:          搜索数量
    # pattern:      搜索的类型, 合法取值eListBoth, eListDirOnly, eListFileOnly
    # prefix:       搜索前缀
    # context:      搜索上下文
    # order:        搜索顺序(0 正序, 1逆序)
    def __init__(self, bucket_name, cos_path, num=199, 
            pattern = u'eListBoth', prefix=u'', context=u'', order=0):
        super(ListFolderRequest, self).__init__(bucket_name, cos_path)
        self._num     = num
        self._pattern = pattern
        self._prefix  = prefix
        self._context = context
        self._order   = order

    # 设置List数量
    def set_num(self, num):
        self._num = num

    # 获取List数量
    def get_num(self):
        return self._num

    # 获取List类型
    def set_pattern(self, pattern):
        self._pattern = pattern

    # 获取List类型
    def get_pattern(self):
        return self._pattern

    # 设置前缀
    def set_prefix(self, prefix):
        self._preifx = prefix
        
    # 获取前缀
    def get_prefix(self):
        return self._prefix

    # 设置搜索上下文
    def set_context(self, context):
        self._context = context
    
    # 获取搜索上下文
    def get_context(self):
        return self._context

    # 设置搜索顺序
    def set_order(self, order=0):
        self._order = order

    # 获取搜索顺序
    def get_order(self):
        return self._order

    # 检查参数是否有效 
    def check_params_valid(self):
        if not super(ListFolderRequest, self).check_params_valid():
            return False
        if not self._param_check.check_cos_path_valid(self._cos_path,
                is_file_path=False):
            return False
        if not self._param_check.check_param_unicode('prefix', self._prefix):
            return False
        if not self._param_check.check_param_unicode('context', self._context):
            return False
        if not self._param_check.check_list_order(self._order):
            return False
        return self._param_check.check_list_pattern(self._pattern)



#!/usr/bin/env python
# coding=utf-8
from .cos_client  import CosClient
from .cos_client  import CosConfig
from .cos_client  import CredInfo
from .cos_request import UploadFileRequest
from .cos_request import UploadSliceFileRequest
from .cos_request import UpdateFileRequest
from .cos_request import UpdateFolderRequest
from .cos_request import DelFolderRequest
from .cos_request import DelFileRequest
from .cos_request import CreateFolderRequest
from .cos_request import StatFileRequest
from .cos_request import StatFolderRequest
from .cos_request import MoveFileRequest
from .cos_request import ListFolderRequest
from .cos_auth    import Auth
from .cos_cred    import CredInfo 



#!/usr/bin/env python
# coding=utf-8

import requests
from cos_cred import CredInfo
from cos_config import CosConfig
from cos_op import FileOp
from cos_op import FolderOp
from cos_request import UploadFileRequest
from cos_request import UploadSliceFileRequest
from cos_request import UpdateFileRequest
from cos_request import UpdateFolderRequest
from cos_request import DelFileRequest
from cos_request import MoveFileRequest
from cos_request import DelFolderRequest
from cos_request import CreateFolderRequest
from cos_request import StatFolderRequest
from cos_request import StatFileRequest
from cos_request import ListFolderRequest
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

################################################################################
# Cos客户端类                                                                  #
################################################################################
class CosClient(object):
    # 设置用户的相关信息
    def __init__(self, appid, secret_id, secret_key):
        self._cred         = CredInfo(appid, secret_id, secret_key)
        self._config       = CosConfig()
        self._http_session = requests.session()
        self._file_op      = FileOp(self._cred, self._config, self._http_session)
        self._folder_op    = FolderOp(self._cred, self._config, self._http_session)

    # 设置config
    def set_config(self, config):
        assert isinstance(config, CosConfig)
        self._config = config
        self._file_op.set_config(config)
        self._folder_op.set_config(config)

    # 获取config
    def get_config(self):
        return self._config

    # 设置用户的身份信息
    def set_cred(self, cred):
        assert isinstance(cred, CredInfo)
        self._cred = cred
        self._file_op.set_cred(cred)
        self._folder_op.set_cred(cred)

    # 获取用户的相关信息
    def get_cred(self):
        return self._cred

    # 上传文件(自动根据文件大小，选择上传策略, 强烈推荐使用)
    # 上传策略: 8MB以下适用单文件上传, 8MB(含)适用分片上传
    def upload_file(self, request):
        assert isinstance(request, UploadFileRequest)
        return self._file_op.upload_file(request)

    # 单文件上传接口, 适用用小文件8MB以下
    # 最大不得超过20MB, 否则会返回参数错误
    def upload_single_file(self, request):
        assert isinstance(request, UploadFileRequest)
        return self._file_op.upload_single_file(request)

    # 分片上传接口, 适用于大文件8MB及以上
    def upload_slice_file(self, request):
        assert isinstance(request, UploadSliceFileRequest)
        return self._file_op.upload_slice_file(request)

    # 删除文件
    def del_file(self, request):
        assert isinstance(request, DelFileRequest)
        return self._file_op.del_file(request)

    # 获取文件属性
    def stat_file(self, request):
        assert isinstance(request, StatFileRequest)
        return self._file_op.stat_file(request)

    # 更新文件属性
    def update_file(self, request):
        assert isinstance(request, UpdateFileRequest)
        return self._file_op.update_file(request)

    # 移动文件
    def move_file(self, request):
        assert isinstance(request, MoveFileRequest)
        return self._file_op.move_file(request)

    # 创建目录
    def create_folder(self, request):
        assert isinstance(request, CreateFolderRequest)
        return self._folder_op.create_folder(request)

    # 删除目录
    def del_folder(self, request):
        assert isinstance(request, DelFolderRequest)
        return self._folder_op.del_folder(request)

    # 获取folder属性请求
    def stat_folder(self, request):
        assert isinstance(request, StatFolderRequest)
        return self._folder_op.stat_folder(request)
    
    # 更新目录属性
    def update_folder(self, request):
        assert isinstance(request, UpdateFolderRequest)
        return self._folder_op.update_folder(request)

    # 获取目录下的文件和目录列表
    def list_folder(self, request):
        assert isinstance(request, ListFolderRequest)
        return self._folder_op.list_folder(request)



#!/usr/bin/env python
# coding=utf-8

import os
import time
import json
import hashlib
import urllib
import cos_auth
import cos_config
from cos_err     import CosErr
from cos_request import UploadFileRequest
from cos_request import UploadSliceFileRequest
from cos_request import UpdateFileRequest
from cos_request import DelFileRequest
from cos_request import StatFileRequest
from cos_request import MoveFileRequest
from cos_request import CreateFolderRequest
from cos_request import UpdateFolderRequest
from cos_request import StatFolderRequest
from cos_request import DelFolderRequest
from cos_request import ListFolderRequest

################################################################################
# BaseOp基本操作类型                                                           #
################################################################################
class BaseOp(object):
    # cred: 用户的身份信息
    # config: cos_config配置类
    # http_session: http 会话
    # expired_period: 签名过期时间, 单位秒
    def __init__(self, cred, config, http_session):
        self._cred           = cred
        self._config         = config
        self._http_session   = http_session
        self._expired_period = self._config.get_sign_expired()

    # 设置用户的身份信息
    def set_cred(self, cred):
        self._cred = cred

    # 设置config
    def set_config(self, config):
        self._config = config
        self._expired_period = self._config.get_sign_expired()

    # 生成url
    def _build_url(self, bucket, cos_path):
        bucket    = bucket.encode('utf8')
        end_point = self._config.get_end_point().rstrip('/').encode('utf8')
        appid     = self._cred.get_appid()
        cos_path  = urllib.quote(cos_path.encode('utf8'), '~/')
        url       = '%s/%s/%s%s' % (end_point, appid, bucket, cos_path)
        return url

    # 发送http请求
    def send_request(self, method, bucket, cos_path, **args):
        url = self._build_url(bucket, cos_path)
        http_resp = {}
        try:
            if method == 'POST':
                http_resp = self._http_session.post(url, verify=False, **args)
            else:
                http_resp = self._http_session.get(url, verify=False, **args)

            status_code = http_resp.status_code
            if (status_code == 200 or status_code == 400):
                return http_resp.json()
            else:
                err_detail = 'url:%s, status_code:%d' % (url, status_code)
                return CosErr.get_err_msg(CosErr.NETWORK_ERROR, err_detail)
        except Exception as e:
            err_detail = 'url:%s, exception:%s' % (url, repr(e))
            return CosErr.get_err_msg(CosErr.SERVER_ERROR, err_detail)

    # 检查用户输入参数, 检查通过返回None, 否则返回一个代表错误原因的dict
    def _check_params(self, request):
        if not self._cred.check_params_valid():
            return CosErr.get_err_msg(CosErr.PARAMS_ERROR, self._cred.get_err_tips())
        if not request.check_params_valid():
            return CosErr.get_err_msg(CosErr.PARAMS_ERROR, request.get_err_tips())
        return None

    # 删除文件或者目录, is_file_op为True表示是文件操作
    def del_base(self, request):
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        auth     = cos_auth.Auth(self._cred)
        bucket   = request.get_bucket_name()
        cos_path = request.get_cos_path()
        sign     = auth.sign_once(bucket, cos_path)

        http_header                  = {}
        http_header['Authorization'] = sign
        http_header['Content-Type']  = 'application/json'
        http_header['User-Agent']    = self._config.get_user_agent()

        http_body       = {}
        http_body['op'] = 'delete'

        timeout = self._config.get_timeout()
        return self.send_request('POST', bucket, cos_path, headers=http_header,
                data=json.dumps(http_body), timeout=timeout)

    # 获取文件和目录的属性
    def stat_base(self, request):
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        auth     = cos_auth.Auth(self._cred)
        bucket   = request.get_bucket_name()
        cos_path = request.get_cos_path()
        expired  = int(time.time()) + self._expired_period
        sign     = auth.sign_more(bucket, cos_path, expired)

        http_header                  = {}
        http_header['Authorization'] = sign
        http_header['User-Agent']    = self._config.get_user_agent()

        http_body       = {}
        http_body['op'] = 'stat'

        timeout = self._config.get_timeout()
        return self.send_request('GET', bucket, cos_path, headers=http_header,
                params=http_body, timeout=timeout)

################################################################################
# FileOp 文件相关操作                                                          #
################################################################################
class FileOp(BaseOp):
    # cred: 用户的身份信息
    # config: cos_config配置类
    # http_session: http 会话
    def __init__(self, cred, config, http_session):
        BaseOp.__init__(self, cred, config, http_session)
        # 单文件上传的最大上限是20MB
        self.max_single_file = 20 * 1024 * 1024

    # 获取content的sha1
    def _sha1_content(self, content):
        sha1_obj = hashlib.sha1()
        sha1_obj.update(content) 
        return sha1_obj.hexdigest()

    # 获取文件的sha1
    def _sha1_file(self, file_path):
        sha1_obj = hashlib.sha1()
        with open(file_path, 'rb') as f:
            while True:
                block_data = f.read(4096)
                if not block_data:
                    break
                sha1_obj.update(block_data)
        return sha1_obj.hexdigest()

    # 更新文件
    def update_file(self, request):
        assert isinstance(request, UpdateFileRequest)
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        auth     = cos_auth.Auth(self._cred)
        bucket   = request.get_bucket_name()
        cos_path = request.get_cos_path()
        sign     = auth.sign_once(bucket, cos_path)

        http_header                  = {}
        http_header['Authorization'] = sign
        http_header['Content-Type']  = 'application/json'
        http_header['User-Agent']    = self._config.get_user_agent()

        http_body             = {}
        http_body['op']       = 'update'

        update_flag = request.get_flag()
        http_body['flag'] = str(update_flag)
        if (update_flag & 0x01 != 0):
            http_body['biz_attr'] = request.get_biz_attr()
        if (update_flag & 0x80 != 0):
            http_body['authority'] = request.get_authority()
        if (update_flag & 0x40 != 0):
            http_body['custom_headers'] = request.get_custom_headers()

        timeout = self._config.get_timeout()
        return self.send_request('POST', bucket, cos_path, headers=http_header, 
                data=json.dumps(http_body), timeout=timeout)

    # 删除文件
    def del_file(self, request):
        assert isinstance(request, DelFileRequest)
        return self.del_base(request)

    # 获取文件的属性
    def stat_file(self, request):
        assert isinstance(request, StatFileRequest)
        return self.stat_base(request)

    # 移动文件
    def move_file(self, request):
        assert isinstance(request, MoveFileRequest)
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        auth     = cos_auth.Auth(self._cred)
        bucket   = request.get_bucket_name()
        cos_path = request.get_cos_path()
        expired  = int(time.time()) + self._expired_period
        sign     = auth.sign_more(bucket, cos_path, expired)

        http_header                  = {}
        http_header['Authorization'] = sign
        http_header['Content-Type']  = 'application/json'
        http_header['User-Agent']    = self._config.get_user_agent()

        http_body                  = {}
        http_body['op']            = 'move'
        http_body['dest_fileid']   = request.get_dst_cos_path()
        http_body['to_over_write'] = request.get_over_write()

        timeout = self._config.get_timeout()
        return self.send_request('POST', bucket, cos_path, headers=http_header, 
                data=json.dumps(http_body), timeout=timeout)

    # 上传文件, 根据用户的文件大小,选择单文件上传和分片上传策略
    def upload_file(self, request):
        assert isinstance(request, UploadFileRequest)
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        local_path = request.get_local_path()
        file_size  = os.path.getsize(local_path)

        suit_single_file_zie = 8 * 1024 * 1024
        if (file_size < suit_single_file_zie):
            return self.upload_single_file(request)
        else:
            bucket     = request.get_bucket_name()
            cos_path   = request.get_cos_path()
            local_path = request.get_local_path()
            slice_size = 512 * 1024
            biz_attr   = request.get_biz_attr()
            upload_slice_request = UploadSliceFileRequest(bucket, cos_path,
                    local_path, slice_size, biz_attr)
            return self.upload_slice_file(upload_slice_request)

    # 单文件上传
    def upload_single_file(self, request):
        assert isinstance(request, UploadFileRequest)
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        local_path = request.get_local_path()
        file_size = os.path.getsize(local_path)
        # 判断文件是否超过单文件最大上限, 如果超过则返回错误
        # 并提示用户使用别的接口
        if file_size > self.max_single_file:
            return CosErr.get_err_msg(CosErr.NETWORK_ERROR, 
                    'file is to big, please use upload_file interface')

        auth     = cos_auth.Auth(self._cred)
        bucket   = request.get_bucket_name()
        cos_path = request.get_cos_path()
        expired  = int(time.time()) + self._expired_period
        sign     = auth.sign_more(bucket, cos_path, expired)

        http_header = {}
        http_header['Authorization'] = sign
        http_header['User-Agent'] = self._config.get_user_agent()

        with open(local_path, 'rb') as f:
            file_content = f.read()

        http_body                = {}
        http_body['op']          = 'upload'
        http_body['filecontent'] = file_content
        http_body['sha']         = self._sha1_content(file_content)
        http_body['biz_attr']    = request.get_biz_attr()
        http_body['insertOnly']  = str(request.get_insert_only())

        timeout = self._config.get_timeout()
        return self.send_request('POST', bucket, cos_path, headers=http_header, 
                files=http_body, timeout=timeout)

    # 分片文件上传(串行)
    def upload_slice_file(self, request):
        assert isinstance(request, UploadSliceFileRequest)
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        control_ret = self._upload_slice_control(request)
        # 表示控制分片已经产生错误信息
        if control_ret[u'code'] != 0:
            return control_ret
        # 命中秒传
        if control_ret[u'data'].has_key(u'access_url'):
            return control_ret

        bucket     = request.get_bucket_name()
        cos_path   = request.get_cos_path()
        local_path = request.get_local_path()
        file_size  = os.path.getsize(local_path)
        slice_size = control_ret[u'data'][u'slice_size']
        offset     = control_ret[u'data'][u'offset']
        session    = control_ret[u'data'][u'session']

        with open(local_path, 'rb') as local_file:
            local_file.seek(offset)
            while offset < file_size:
                file_content = local_file.read(slice_size)
                retry_count = 0
                max_retry = 3
                # 如果分片数据上传发生错误, 则进行重试,默认3次
                while retry_count < max_retry:
                    data_ret = self._upload_slice_data(bucket, cos_path, 
                            file_content, session, offset)
                    if data_ret[u'code'] == 0:
                        if data_ret[u'data'].has_key(u'access_url'):
                            return data_ret
                        else:
                            break
                    else:
                        retry_count += 1

                if retry_count == max_retry:
                    return data_ret
                else:
                    offset += slice_size
        return data_ret

    # 串行分片第一步, 上传控制分片
    def _upload_slice_control(self, request):
        auth     = cos_auth.Auth(self._cred)
        bucket   = request.get_bucket_name()
        cos_path = request.get_cos_path()
        expired  = int(time.time()) + self._expired_period
        sign     = auth.sign_more(bucket, cos_path, expired)

        http_header                  = {}
        http_header['Authorization'] = sign
        http_header['User-Agent']    = self._config.get_user_agent()

        local_path  = request.get_local_path()
        sha1_digest = self._sha1_file(local_path)
        file_size   = os.path.getsize(local_path)
        slice_size  = request.get_slice_size()
        biz_atrr    = request.get_biz_attr()

        http_body               = {}
        http_body['op']         = 'upload_slice'
        http_body['sha']        = sha1_digest
        http_body['filesize']   = str(file_size)
        http_body['slice_size'] = str(slice_size)
        http_body['biz_attr']   = request.get_biz_attr()
        http_body['insertOnly'] = str(request.get_insert_only())

        timeout = self._config.get_timeout()
        return self.send_request('POST', bucket, cos_path, headers=http_header, 
                files=http_body, timeout=timeout)

    # 串行分片第二步, 上传数据分片
    def _upload_slice_data(self, bucket, cos_path, file_content, session, offset):
        auth     = cos_auth.Auth(self._cred)
        expired  = int(time.time()) + self._expired_period
        sign     = auth.sign_more(bucket, cos_path, expired)

        http_header                  = {}
        http_header['Authorization'] = sign
        http_header['User-Agent']    = self._config.get_user_agent()

        http_body                = {}
        http_body['op']          = 'upload_slice'
        http_body['filecontent'] = file_content
        http_body['session']     = session
        http_body['offset']      = str(offset)

        timeout = self._config.get_timeout()
        return self.send_request('POST', bucket, cos_path, headers=http_header, 
                files=http_body, timeout=timeout)

################################################################################
# FolderOp 目录相关操作                                                        #
################################################################################
class FolderOp(BaseOp):
    def __init__(self, cred, config, http_session):
        BaseOp.__init__(self, cred, config, http_session)

    # 更新目录
    def update_folder(self, request):
        assert isinstance(request, UpdateFolderRequest)
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        auth     = cos_auth.Auth(self._cred)
        bucket   = request.get_bucket_name()
        cos_path = request.get_cos_path()
        sign     = auth.sign_once(bucket, cos_path)

        http_header                  = {}
        http_header['Authorization'] = sign
        http_header['Content-Type']  = 'application/json'
        http_header['User-Agent']    = self._config.get_user_agent()

        http_body             = {}
        http_body['op']       = 'update'
        http_body['biz_attr'] = request.get_biz_attr()

        timeout = self._config.get_timeout()
        return self.send_request('POST', bucket, cos_path, headers=http_header, 
                data=json.dumps(http_body), timeout=timeout)

    # 删除目录
    def del_folder(self, request):
        assert isinstance(request, DelFolderRequest)
        return self.del_base(request)

    # 获取目录属性
    def stat_folder(self, request):
        assert isinstance(request, StatFolderRequest)
        return self.stat_base(request)

    # 创建目录
    def create_folder(self, request):
        assert isinstance(request, CreateFolderRequest)
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        auth     = cos_auth.Auth(self._cred)
        bucket   = request.get_bucket_name()
        cos_path = request.get_cos_path()
        expired  = int(time.time()) + self._expired_period
        sign     = auth.sign_more(bucket, cos_path, expired)

        http_header = {}
        http_header['Authorization'] = sign
        http_header['Content-Type'] = 'application/json'
        http_header['User-Agent'] = self._config.get_user_agent()

        http_body             = {}
        http_body['op']       = 'create'
        http_body['biz_attr'] = request.get_biz_attr()

        timeout = self._config.get_timeout()
        return self.send_request('POST', bucket, cos_path, headers=http_header, 
                data=json.dumps(http_body), timeout=timeout)

    # list目录
    def list_folder(self, request):
        assert isinstance(request, ListFolderRequest)
        check_params_ret = self._check_params(request)
        if check_params_ret != None:
            return check_params_ret

        http_body            = {}
        http_body['op']      = 'list'
        http_body['num']     = request.get_num()
        http_body['pattern'] = request.get_pattern()
        http_body['order']   = request.get_order()
        http_body['context'] = request.get_context()
        http_body['prefix']  = request.get_prefix()

        auth     = cos_auth.Auth(self._cred)
        bucket   = request.get_bucket_name()
        list_path = request.get_cos_path() + request.get_prefix()
        expired  = int(time.time()) + self._expired_period
        sign     = auth.sign_more(bucket, list_path, expired)

        http_header                  = {}
        http_header['Authorization'] = sign
        http_header['User-Agent']    = self._config.get_user_agent()

        timeout = self._config.get_timeout()
        return self.send_request('GET', bucket, list_path, headers=http_header, 
                params=http_body, timeout=timeout)



#!/usr/bin/env python
# coding=utf-8

import random
import time
import urllib
import hmac
import hashlib
import binascii
import base64
import cos_cred

class Auth(object):
    def __init__(self, cred):
        self.cred = cred

    def app_sign(self, bucket, cos_path, expired, upload_sign=True):
        appid      = self.cred.get_appid()
        bucket     = bucket.encode('utf8')
        secret_id  = self.cred.get_secret_id().encode('utf8')
        now        = int(time.time())
        rdm        = random.randint(0, 999999999)
        cos_path   = urllib.quote(cos_path.encode('utf8'), '~/')
        if upload_sign:
            fileid = '/%s/%s%s' % (appid, bucket, cos_path)
        else:
            fileid = cos_path
        sign_tuple = (appid, secret_id, expired, now, rdm, fileid, bucket)

        plain_text  = 'a=%s&k=%s&e=%d&t=%d&r=%d&f=%s&b=%s' % sign_tuple
        secret_key  = self.cred.get_secret_key().encode('utf8')
        sha1_hmac   = hmac.new(secret_key, plain_text, hashlib.sha1)
        hmac_digest = sha1_hmac.hexdigest()
        hmac_digest = binascii.unhexlify(hmac_digest)
        sign_hex    = hmac_digest + plain_text
        sign_base64 = base64.b64encode(sign_hex)
        return sign_base64

    # 单次签名(针对删除和更新操作)
    # bucket: bucket名称
    # cos_path: 要操作的cos路径, 以'/'开始
    def sign_once(self, bucket, cos_path):
        return self.app_sign(bucket, cos_path, 0)

    # 多次签名(针对上传文件，创建目录, 获取文件目录属性, 拉取目录列表)
    # bucket: bucket名称
    # cos_path: 要操作的cos路径, 以'/'开始
    # expired:  签名过期时间, UNIX时间戳
    #           如想让签名在30秒后过期, 即可将expired设成当前时间加上30秒
    def sign_more(self, bucket, cos_path, expired):
        return self.app_sign(bucket, cos_path, expired)

    # 下载签名(用于获取后拼接成下载链接，下载私有bucket的文件)
    # bucket: bucket名称
    # cos_path: 要下载的cos文件路径, 以'/'开始
    # expired:  签名过期时间, UNIX时间戳
    #           如想让签名在30秒后过期, 即可将expired设成当前时间加上30秒
    def sign_download(self, bucket, cos_path, expired):
        return self.app_sign(bucket, cos_path, expired, False)



#!/usr/bin/env python
# coding=utf-8

################################################################################
# CosConfig 有关cos的配置
################################################################################
class CosConfig(object):
    def __init__(self):
        self._end_point    = 'http://web.file.myqcloud.com/files/v1'
        self._user_agent   = 'cos-python-sdk-v3.3'
        self._timeout      = 3 
        self._sign_expired = 300

    # 设置COS的域名地址
    def set_end_point(self, end_point):
        self._end_point = end_point

    # 获取域名地址
    def get_end_point(self):
        return self._end_point

    # 获取HTTP头中的user_agent
    def get_user_agent(self):
        return self._user_agent

    # 设置连接超时, 单位秒
    def set_timeout(self, time_out):
        assert isinstance(time_out, int)
        self._timeout = time_out

    # 获取连接超时，单位秒
    def get_timeout(self):
        return self._timeout

    # 设置签名过期时间, 单位秒
    def set_sign_expired(self, expired):
        assert isinstance(expired, int)
        self._sign_expired = expired

    # 获取签名过期时间, 单位秒
    def get_sign_expired(self):
        return self._sign_expired

    # 打开https
    def enable_https(self):
        self._end_point = 'https://web.file.myqcloud.com/files/v1'
